from sqlmodel import SQLModel, Field
from datetime import datetime
from typing import Optional
from decimal import Decimal


class SysMdStrategy(SQLModel, table=True):
    __tablename__ = "sys_md_strategy"
    """马丁策略表"""
    strategy_id: Optional[int] = Field(default=None, primary_key=True, description='策略id')
    user_id: Optional[int] = Field(default=None, description='用户id')
    name: Optional[str] = Field(default=None, max_length=2550, description='策略名称')
    position_side: Optional[str] = Field(default=None, max_length=10, description='马丁持仓方向。LONG:做多，SHORT:做空 BOTH:双向')
    open_high_order_reset_take_profit: Optional[bool] = Field(default=None, description='高单重挂止盈开关')
    when_repull_take_profit: Optional[int] = Field(default=None, description='第几单后重挂止盈')
    repull_take_profit_num: Optional[Decimal] = Field(default=Decimal(0), description='重挂止盈大小')

    long_1st_pos_val: Optional[Decimal] = Field(default=Decimal(0), description='做多首单价值')
    long_cover_1st_pos_val: Optional[Decimal] = Field(default=Decimal(0), description='做多补单的首单价值')
    long_act_order_num: Optional[int] = Field(default=None, description='做多活跃订单数量')
    long_cover_order_num: Optional[int] = Field(default=None, description='做多补单次数')
    long_bdcz: Optional[int] = Field(default=0, max_length=25, description='做多波动参照')
    long_bdbd: Optional[Decimal] = Field(default=Decimal(0), description='做多补单波动')
    long_cover_value_mult: Optional[Decimal] = Field(default=Decimal(0), description='做多补单倍数')
    long_accel_cover: Optional[Decimal] = Field(default=Decimal(0), description='做多加速补单')
    long_take_profit_type: Optional[str] = Field(default=None, max_length=25, description='做多止盈方式 price:固定价格止盈 percent:百分比止盈')
    long_take_profit_price: Optional[Decimal] = Field(default=Decimal(0), description='做多止盈价格')
    long_take_profit_percent: Optional[Decimal] = Field(default=Decimal(0), description='做多止盈百分比')
    long_dj_open: Optional[bool] = Field(default=None, description='做多叠加补单')
    long_dj_cover_1st_pos_val: Optional[Decimal] = Field(default=Decimal(0), description='做多叠加补单首单')
    long_dj_cover_order_num: Optional[int] = Field(default=None, description='做多叠加补单次数')
    long_dj_bdcz: Optional[int] = Field(default=0, max_length=25, description='做多叠加波动参照')
    long_dj_bdbd: Optional[Decimal] = Field(default=Decimal(0), description='做多叠加补单波动')
    long_dj_cover_value_mult: Optional[Decimal] = Field(default=Decimal(0), description='做多叠加补单倍数')
    long_dj_accel_cover: Decimal = Field(default=Decimal(0), description='做多叠加加速补单')
    long_dj_take_profit_type: Optional[str] = Field(default=None, max_length=25, description='做多叠加止盈方式 price:固定价格止盈 percent:百分比止盈')
    long_dj_take_profit_price: Optional[Decimal] = Field(default=Decimal(0), description='做多叠加止盈价格')
    long_dj_take_profit_percent: Optional[Decimal] = Field(default=Decimal(0), description='做多叠加止盈百分比')
    long_stop_loss: Optional[bool] = Field(default=None, description='做多止损开关')
    long_stop_loss_wait_time: Optional[int] = Field(default=None, description='做多止损前等待时间 秒')
    long_stop_loss_usdt: Optional[Decimal] = Field(default=Decimal(0), description='做多浮亏多少止损')

    short_1st_pos_val: Optional[Decimal] = Field(default=Decimal(0), description='做空首单价值')
    short_cover_1st_pos_val: Optional[Decimal] = Field(default=Decimal(0), description='做空补单的首单价值')
    short_act_order_num: Optional[int] = Field(default=None, description='做空活跃订单数量')
    short_cover_order_num: Optional[int] = Field(default=None, description='做空补单次数')
    short_bdcz: Optional[int] = Field(default=0, max_length=25, description='做空波动参照')
    short_bdbd: Optional[Decimal] = Field(default=Decimal(0), description='做空补单波动')
    short_cover_value_mult: Optional[Decimal] = Field(default=Decimal(0), description='做空补单倍数')
    short_accel_cover: Optional[Decimal] = Field(default=Decimal(0), description='做空加速补单')
    short_take_profit_type: Optional[str] = Field(default=None, max_length=25, description='做空止盈方式 price:固定价格止盈 percent:百分比止盈')
    short_take_profit_price: Optional[Decimal] = Field(default=Decimal(0), description='做空止盈价格')
    short_take_profit_percent: Optional[Decimal] = Field(default=Decimal(0), description='做空止盈百分比')
    short_dj_open: Optional[bool] = Field(default=None, description='做空叠加补单')
    short_dj_cover_1st_pos_val: Optional[Decimal] = Field(default=Decimal(0), description='做空叠加补单首单')
    short_dj_cover_order_num: Optional[int] = Field(default=None, description='做空叠加补单次数')
    short_dj_bdcz: Optional[int] = Field(default=0, max_length=25, description='做空叠加波动参照')
    short_dj_bdbd: Optional[Decimal] = Field(default=Decimal(0), description='做空叠加补单波动')
    short_dj_cover_value_mult: Optional[Decimal] = Field(default=Decimal(0), description='做空叠加补单倍数')
    short_dj_accel_cover: Decimal = Field(default=Decimal(0), description='做空叠加加速补单')
    short_dj_take_profit_type: Optional[str] = Field(default=None, max_length=25, description='做空叠加止盈方式 price:固定价格止盈 percent:百分比止盈')
    short_dj_take_profit_price: Optional[Decimal] = Field(default=Decimal(0), description='做空叠加止盈价格')
    short_dj_take_profit_percent: Optional[Decimal] = Field(default=Decimal(0), description='做空叠加止盈百分比')
    short_stop_loss: Optional[bool] = Field(default=None, description='做空止损开关')
    short_stop_loss_wait_time: Optional[int] = Field(default=None, description='做空止损前等待时间 秒')
    short_stop_loss_usdt: Optional[Decimal] = Field(default=Decimal(0), description='做空浮亏多少止损')

    create_time: datetime = Field(default_factory=datetime.now, description='创建时间')
    update_time: datetime = Field(default_factory=datetime.now, description='更新时间')
    is_deleted: bool = Field(default=False, description='是否逻辑删除 0 否 1是')
    is_ban: bool = Field(default=False, description='是否禁用 0 否 1是')
    is_common: Optional[bool] = Field(default=None, description='是否公共的 0 否 1是')
    is_share: Optional[bool] = Field(default=None, description='是否共享 0 否 1是')


class SysMdStrategyRun(SQLModel, table=True):
    __tablename__ = "sys_md_strategy_run"
    """马丁策略运行总览表"""
    run_id: int = Field(primary_key=True, description='运行id')
    user_id: int = Field(description='用户id')
    strategy_id: int = Field(description='策略id')
    exchange_id: int = Field(description='交易所id')
    trade_type: Optional[str] = Field(default=None, max_length=64, description='交易类型  features 合约 spot 现货')
    symbol: Optional[str] = Field(default=None, max_length=64, description='交易对')
    usdt_balance_snapshot: Optional[Decimal] = Field(default=Decimal(0), description='usdt余额快照')
    stop_profit_num: Optional[int] = Field(default=0, description='止盈次数')
    total_stop_profit: Optional[Decimal] = Field(default=Decimal(0), description='止盈总利润')
    fee: Optional[Decimal] = Field(default=Decimal(0), description='总手续费')
    long_stop_profit_num: Optional[int] = Field(default=0, description='做多止盈次数')
    long_total_stop_profit: Optional[Decimal] = Field(default=Decimal(0), description='做多止盈总利润')
    long_fee: Optional[Decimal] = Field(default=Decimal(0), description='做多总手续费')
    long_status: Optional[int] = Field(default=-1, description='做多运行状态 已停止 = 0 运行中 = 1 暂停中 = 2')
    short_stop_profit_num: Optional[int] = Field(default=0, description='做空止盈次数')
    short_total_stop_profit: Optional[Decimal] = Field(default=Decimal(0), description='做空止盈总利润')
    short_fee: Optional[Decimal] = Field(default=Decimal(0), description='做空总手续费')
    short_status: Optional[int] = Field(default=-1, description='做空运行状态 已停止 = 0 运行中 = 1 暂停中 = 2')
    start_time: Optional[datetime] = Field(default=None, description='开始时间')
    end_time: Optional[datetime] = Field(default=None, description='结束时间')
    remark: Optional[str] = Field(default=None, max_length=2550, description='交易对')
    create_time: datetime = Field(default_factory=datetime.now, description='创建时间')
    update_time: datetime = Field(default_factory=datetime.now, description='更新时间')
    is_deleted: bool = Field(default=False, description='是否逻辑删除 0 否 1是')


class SysMdStrategyRunDetail(SQLModel, table=True):
    __tablename__ = "sys_md_strategy_run_detail"
    """马丁策略运行明细表"""
    run_detail_id: int = Field(primary_key=True, description='运行明细id')
    run_id: int = Field(description='运行id')
    user_id: int = Field(description='用户id')
    strategy_id: int = Field(description='策略id')
    exchange_id: int = Field(description='交易所id')
    symbol: Optional[str] = Field(default=None, max_length=64, description='交易对')
    position_side: Optional[str] = Field(default=None, max_length=64, description='持仓方向    LONG SHORT')
    run_index: Optional[int] = Field(default=0, description='当前第几轮')
    cover_num: Optional[int] = Field(default=0, description='已补订单数量')
    open_order_num: Optional[Decimal] = Field(default=Decimal(0), description='开单数量')
    open_order_price: Optional[Decimal] = Field(default=Decimal(0), description='开单价格')
    max_position_num: Optional[Decimal] = Field(default=Decimal(0), description='最大仓位数量')
    max_position_price: Optional[Decimal] = Field(default=Decimal(0), description='最大仓位价格')
    max_position_value: Optional[Decimal] = Field(default=Decimal(0), description='最大仓位价值')
    stop_profit_num: Optional[Decimal] = Field(default=Decimal(0), description='止盈数量')
    stop_profit_price: Optional[Decimal] = Field(default=Decimal(0), description='止盈价格')
    stop_profit_value: Optional[Decimal] = Field(default=Decimal(0), description='止盈利润')
    fee: Optional[Decimal] = Field(default=Decimal(0), description='手续费')
    run_time: Optional[int] = Field(default=None, description='运行时长 分钟')
    start_time: Optional[datetime] = Field(default=None, description='开始时间')
    end_time: Optional[datetime] = Field(default=None, description='结束时间')
    create_time: datetime = Field(default_factory=datetime.now, description='创建时间')
    update_time: datetime = Field(default_factory=datetime.now, description='更新时间')
    is_deleted: bool = Field(default=False, description='是否逻辑删除 0 否 1是')



class SysMdStrategyRunRealTime(SQLModel, table=True):
    __tablename__ = "sys_md_strategy_run_real_time"
    """马丁策略运行实时信息表"""

    run_real_time_id: int = Field(primary_key=True, description='实时信息id')
    run_detail_id: int = Field(description='运行明细id')
    run_id: int = Field(description='运行id')
    user_id: int = Field(description='用户id')
    strategy_id: int = Field(description='策略id')
    exchange_id: int = Field(description='交易所id')
    symbol: Optional[str] = Field(default=None, max_length=64, description='交易对')
    position_side: Optional[str] = Field(default=None, max_length=64, description='持仓方向    LONG SHORT')
    run_index: Optional[int] = Field(default=0, description='当前第几轮')
    cover_num: Optional[int] = Field(default=0, description='已补订单数量')
    open_order_price: Optional[Decimal] = Field(default=Decimal(0), description='开单价格')
    position_price: Optional[Decimal] = Field(default=Decimal(0), description='仓位价格')
    position_num: Optional[Decimal] = Field(default=Decimal(0), description='仓位数量')
    position_value: Optional[Decimal] = Field(default=Decimal(0), description='仓位价值')
    real_time_price: Optional[Decimal] = Field(default=Decimal(0), description='实时价格')
    stop_profit_price: Optional[Decimal] = Field(default=Decimal(0), description='止盈价格')
    limit_price: Optional[Decimal] = Field(default=Decimal(0), description='极限价格')
    how_much_usdt_can_stop_profit: Optional[Decimal] = Field(default=Decimal(0), description='差多少u止盈')
    run_time: Optional[int] = Field(default=0, description='运行时长 分钟')
    start_time: Optional[datetime] = Field(default=None, description='开始时间')
    end_time: Optional[datetime] = Field(default=None, description='结束时间')
    create_time: datetime = Field(default_factory=datetime.now, description='创建时间')
    update_time: datetime = Field(default_factory=datetime.now, description='更新时间')
    is_deleted: bool = Field(default=False, description='是否逻辑删除 0 否 1是')